//+------------------------------------------------------------------+
//|                                                 ParabolicSAR.mq5 |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2009-2017, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
//--- indicator settings
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   2
#property indicator_type1   DRAW_ARROW
#property indicator_color1  clrDodgerBlue
#property indicator_type2   DRAW_ARROW
#property indicator_color2  clrSalmon

//--- External parametrs
input int            inpATRperiod   = 10;       // ATR Period
input double         inpATRfactor   = 1.5;      // ATR Factor
input ENUM_MA_METHOD inpATRmethod   = MODE_SMA; // Average Type

//---- buffers
double               longBuffer[];
double               shortBuffer[];
int                  hndMA;
int                  hndATR;
double               ma[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
{
//---
   SetIndexBuffer(0,longBuffer);
   SetIndexBuffer(1,shortBuffer);
   PlotIndexSetInteger(0,PLOT_ARROW,159);
   PlotIndexSetInteger(1,PLOT_ARROW,159);
   
   hndATR = iATR(_Symbol,0,1);
   hndMA = iMA(_Symbol,0,inpATRperiod,0,inpATRmethod,hndATR);
//----
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   double loss;
   
//--- check for minimum rates count
   if(rates_total<3)
      return(0);
//--- detect current position 
   int pos=prev_calculated-1;
//--- correct position
   if(pos<1)
   {
      pos=1;
      longBuffer[0]=low[0];
      longBuffer[1]=low[1];
      shortBuffer[0]=EMPTY_VALUE;
      shortBuffer[1]=EMPTY_VALUE;
   }
//---main cycle
   CopyBuffer(hndMA,0,0,rates_total,ma);
   for(int i=pos;i<rates_total && !IsStopped();i++)
   {
      if(i<=0) continue;
      longBuffer[i] = shortBuffer[i] = EMPTY_VALUE;
      loss = ma[i]*inpATRfactor;
      
      if(longBuffer[i-1]!=EMPTY_VALUE)
      {
         if(close[i-1]>=longBuffer[i-1]) longBuffer[i] = MathMax(longBuffer[i-1],MathMax(close[i-1],close[i])-loss);
         else shortBuffer[i] = MathMin(close[i-1],close[i])+loss;
      }
      else
      {
         if(close[i-1]<=shortBuffer[i-1]) shortBuffer[i] = MathMin(shortBuffer[i-1],MathMin(close[i-1],close[i])+loss);
         else longBuffer[i] = MathMax(close[i-1],close[i])-loss;
      }
   }
//---- OnCalculate done. Return new prev_calculated.
   return(rates_total);
}


